/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
   comments & bugs to: Stefan.Eilemann@simpack.de */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		/* for cos(), sin(), and sqrt() */
#include <GL/glut.h>
#include "glload.h"
#include <time.h>
#ifndef linux			/* actually only tested on sparc */
#include <sys/param.h>
#include <rpcsvc/rstat.h>
#include <time.h>

#ifdef SVR4

#include <netdb.h>
#include <sys/systeminfo.h>
#include <unistd.h>
#endif /* SVR4 */
#endif /* linux */

extern char hostname[255];
extern struct statstime res;
static char *help_message[] =
{
  "where options include:",
  "    -u <secs>           updatespeed",
  "    -l [0/1]            lighting calculations on/off",
  "    -s [0/1]            load text on the bars on/off",
  "    -p [0/1]            transparent planes on/off",
  "    -c [0/1]            clock on/off",
  "    -h [0/1]            hostname on/off",
  "    -b [0/1/2]          alpha blending on/off/auto",
  "    -a [0/1]            antialiasing on/off",
  "    -t <32x32pic.rgb>   use texturing with this picture",
  "    -tx <int>           texture repeat in x",
  "    -ty <int>           texture repeat in y",
  "    -n count            number of samples/graph",
  "    -r <float>          set red (max) value of small segment to <float> (default 5.0)",
  "    -ss <float>         start of small segment in percent",
  "    -se <float>         end of small segment in percent",
  "    -st <float>         position of text for small segment",
  "    -ms <float>         start of medium segment in percent",
  "    -me <float>         end of medium segment in percent",
  "    -mt <float>         position of text for medium segment",
  "    -ls <float>         start of large segment in percent",
  "    -le <float>         end of large segment in percent",
  "    -lt <float>         position of text for large segment",
  "    -h                  this message",
  NULL
};

#define MIN2( a,b )  ( ( a ) < ( b ) ? ( a ) : ( b ) )
#define MAX2( a,b )  ( ( a ) < ( b ) ? ( b ) : ( a ) )
#define MAX3( a,b,c )  (MAX2 ( a , MAX2(b,c) ))


int updatespeed = 4;		/* default */
int lighting = 1, showLoad = 1, texturing = 0, showPlanes = 1, showClock = 1,
  showHost = 1, xTextureRepeat = 2, yTextureRepeat = 2, blending = 1, highqualitymoving = 0,
  antialias = 1, spinning = 0, moving = 0, anim = 0;	/* state variables, appearance */
GLfloat smallStart = 0.2, smallEnd = 1.0, mediumStart = 0.0, mediumEnd = 0.2,
  largeStart = 0.0, largeEnd = 0.2, smallText = 0.8, mediumText = 0.0,
  largeText = 0.13, redValue = 5.0;	/* state variables, sizes and positions of bars and text */
char *ProgName;
int W = 300, H = 300;
float curquat[4];
float lastquat[4];
int beginx, beginy;
GLdouble bodyWidth = 3.0;
int newModel = 1;
int scaling;
float scalefactor = 1.0;
int numSamples = 100;
float *mesh[3];
int curMeshPointer;
int maxLoad = 0.0;
GLfloat lightZeroPosition[] =
{7.5, 4.0, 20.0, 1.0};
GLfloat lightZeroColor[] =
{0.8, 1.0, 0.8, 1.0};		/* green-tinted */
GLfloat lightOnePosition[] =
{7.5, 4.0, 0.0, 1.0};
GLfloat lightOneColor[] =
{1.0, 0.8, 0.8, 1.0};		/* red-tinted */
GLfloat green[] =
{0.1, 1.0, 0.1, 0.9};
GLfloat red[] =
{1.0, 0.1, 0.1, 0.9};
GLfloat yellow[] =
{1.0, 1.0, 0.1, 0.9};
GLfloat alphagrey[] =
{0.5, 0.5, 0.5, 0.25};
GLubyte *imageDataIn, *imageDataOut;
time_t actualtime;
struct tm *time_struct;
char loctime[20];
long curtime;

const GLubyte stippleMask[128] =
{0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55};
char day_of_week[7][3] =
{"SU", "MO", "TU", "WE", "TH", "FR", "SA"};
char mon_of_year[12][4] =
{"JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"};

/* load texture image */
static void 
imgLoad (char *filename_in, int *w_out, int *h_out, GLubyte ** img_out)
{
  int comp;

  *img_out = (GLubyte *) read_texture (filename_in, w_out, h_out, &comp);
  if (*img_out == NULL) {
    fprintf (stderr, "unable to read %s\n", filename_in);
    exit (1);
  }
  if (comp != 3 && comp != 4) {
    fprintf (stderr, "%s: image is not RGB or RGBA\n", filename_in);
    exit (1);
  }
}
/* recalculate camera position */
void 
recalcModelView (void)
{
  GLfloat m[4][4];

  glPopMatrix ();
  glPushMatrix ();
  build_rotmatrix (m, curquat);
  glMultMatrixf (&m[0][0]);
  if (scalefactor == 1.0) {
    glDisable (GL_NORMALIZE);
  } else {
    glEnable (GL_NORMALIZE);
  }
  glScalef (scalefactor, scalefactor, scalefactor);
  glTranslatef (-8, -8, -bodyWidth / 2);
  newModel = 0;
}

/* displays message at x,y,z in x/y plane*/
void
showMessage (GLfloat x, GLfloat y, GLfloat z, char *message, GLfloat size)
{
  glPushMatrix ();
  if (lighting)
    glDisable (GL_LIGHTING);
  glTranslatef (x, y, z);
  glScalef (.02 * size, .02 * size, .02 * size);
  while (*message) {
    glutStrokeCharacter (GLUT_STROKE_ROMAN, *message);
    message++;
  }
  if (lighting) {
    glEnable (GL_LIGHTING);
    glShadeModel (GL_SMOOTH);
  }
  glPopMatrix ();
}
/* compiles the values of the load matrix to the OpenGL list 1 according to the state variables */
void 
makeLoad ()
{
  char load[10];
  float curmeshval;
  float loaddelta;
  GLfloat n;
  int i;
  float color[4];
  glNewList (1, GL_COMPILE);
  /* first strip, current load, this one is textured or shaded from green to red */
  if (texturing)
    glEnable (GL_TEXTURE_2D);
  glBegin (GL_QUAD_STRIP);
  for (i = 1; i <= numSamples; i++) {
    curmeshval = mesh[0][(curMeshPointer + i) % numSamples];
    loaddelta = curmeshval - mesh[0][(curMeshPointer + i - 1) % numSamples];
    n = M_PI / 2.0 - atan (loaddelta / (float) updatespeed);
    glNormal3f (0, cos (n), sin (n));

    if (!texturing) {
      color[0] = MIN2 (curmeshval / (redValue / 2), 1.0);
      color[1] = MIN2 (MAX2 (2.0 - curmeshval / (redValue / 2), 0.0), 1.0);
      color[2] = 0.0;
      color[3] = 0.75;
      if (lighting)
	glMaterialfv (GL_FRONT, GL_DIFFUSE, color);
      else
	glColor4f (color[0], color[1], color[2], 0.75);
    }
    glTexCoord2f (0.0, (i - 1) / (float) (numSamples - 1) * yTextureRepeat);
    glVertex3f (smallStart * 15.0, 1 + curmeshval * 14.0 / (maxLoad + 1), -10 + (float) (i - 1) / (float) (numSamples - 1) * 20.0);
    glTexCoord2f (xTextureRepeat, (i - 1) / (float) (numSamples - 1) * yTextureRepeat);
    glVertex3f (smallEnd * 15.0, 1 + curmeshval * 14.0 / (maxLoad + 1), -10 + (float) (i - 1) / (float) (numSamples - 1) * 20.0);
  }
  glEnd ();
  if (texturing)
    glDisable (GL_TEXTURE_2D);
  /* second strip, 5min load, yellow */
  if (lighting)
    glMaterialfv (GL_FRONT, GL_DIFFUSE, yellow);
  else
    glColor4f (1.0, 1.0, 0.1, 0.75);
  glBegin (GL_QUAD_STRIP);
  for (i = 1; i <= numSamples; i++) {
    curmeshval = mesh[1][(curMeshPointer + i) % numSamples];
    loaddelta = curmeshval - mesh[1][(curMeshPointer + i - 1) % numSamples];
    n = M_PI / 2.0 - atan (loaddelta / (float) updatespeed);
    glNormal3f (0, cos (n), sin (n));
    glVertex3f (mediumStart * 15.0, 1 + curmeshval * 14.0 / (maxLoad + 1), -10 + (float) (i - 1) / (float) (numSamples - 1) * 20.0);
    glVertex3f (mediumEnd * 15.0, 1 + curmeshval * 14.0 / (maxLoad + 1), -10 + (float) (i - 1) / (float) (numSamples - 1) * 20.0);
  }
  glEnd ();
  /* third strip, 30min load, red */
  if (lighting)
    glMaterialfv (GL_FRONT, GL_DIFFUSE, red);
  else
    glColor4f (1.0, 0.1, 0.1, 0.75);
  glBegin (GL_QUAD_STRIP);
  for (i = 1; i <= numSamples; i++) {
    curmeshval = mesh[2][(curMeshPointer + i) % numSamples];
    loaddelta = curmeshval - mesh[2][(curMeshPointer + i - 1) % numSamples];
    n = M_PI / 2.0 - atan (loaddelta / (float) updatespeed);
    glNormal3f (0, cos (n), sin (n));
    glVertex3f (largeStart * 15.0, 1 + curmeshval * 14.0 / (maxLoad + 1), -10 + (float) (i - 1) / (float) (numSamples - 1) * 20.0);
    glVertex3f (largeEnd * 15.0, 1 + curmeshval * 14.0 / (maxLoad + 1), -10 + (float) (i - 1) / (float) (numSamples - 1) * 20.0);
  }
  glEnd ();
  /* grey planes */
  if (showPlanes) {
    if (lighting)
      glMaterialfv (GL_FRONT, GL_DIFFUSE, alphagrey);
    else
      glColor4f (0.5, 0.5, 0.5, 0.5);
    if (blending != 1) {
      glEnable (GL_POLYGON_STIPPLE);
      glPolygonStipple (stippleMask);
    } else {
      glDisable (GL_POLYGON_STIPPLE);
    }
    glBegin (GL_QUADS);
    for (i = 0; i <= maxLoad + 1; i++) {
      glVertex3f (15.0, (1 + i * 14.0 / (float) (maxLoad + 1)), -10);
      glVertex3f (0.0, (1 + i * 14.0 / (float) (maxLoad + 1)), -10);
      glVertex3f (0.0, (1 + i * 14.0 / (float) (maxLoad + 1)), 10);
      glVertex3f (15.0, (1 + i * 14.0 / (float) (maxLoad + 1)), 10);
    }
    glEnd ();
    if (blending != 1)
      glDisable (GL_POLYGON_STIPPLE);
  }
  /* text fields */
  if (!lighting)
    glColor3f (1.0, 1.0, 1.0);
  if (showLoad) {
    for (i = 0; i <= maxLoad + 1; i++) {
      sprintf (load, "%2.1f", (float) i);
      showMessage (14, (1 + i * 14.0 / (float) (maxLoad + 1)), 10, load, 0.3);
    }
    sprintf (load, "%2.1f", (float) mesh[0][curMeshPointer]);
    showMessage (smallText * 15.0, (1 + mesh[0][curMeshPointer] * 14.0 / (maxLoad + 1)), 10, load, 0.3);
    sprintf (load, "%2.1f", (float) mesh[1][curMeshPointer]);
    showMessage (mediumText * 15.0, (1 + mesh[1][curMeshPointer] * 14.0 / (maxLoad + 1)), 10, load, 0.3);
    sprintf (load, "%2.1f", (float) mesh[2][curMeshPointer]);
    showMessage (largeText * 15.0, (1 + mesh[2][curMeshPointer] * 14.0 / (maxLoad + 1)), 10, load, 0.3);
  }
  if (showHost)
    showMessage (0, 1, 10, hostname, 1.0);

  if (showClock) {
    curtime = time (0);
    time_struct = localtime (&curtime);
    sprintf (loctime, "%.2s,%02d.%.3s %02d:%02d\0"
	     ,day_of_week[time_struct->tm_wday], time_struct->tm_mday, mon_of_year[time_struct->tm_mon],
	     time_struct->tm_hour, time_struct->tm_min);
    showMessage (0, 14.0, 10, loctime, 0.5);
  }
  glEndList ();
}
/* get the actual load values into field mesh[3][numSamples] */
void 
getLoad ()
{
  int i;
  curMeshPointer++;
  curMeshPointer = curMeshPointer % numSamples;
  if (MAX3 (mesh[0][curMeshPointer], mesh[1][curMeshPointer], mesh[2][curMeshPointer]) >= maxLoad) {
    mesh[0][curMeshPointer] = mesh[1][curMeshPointer] = mesh[2][curMeshPointer] = maxLoad = 0.0;
    for (i = 0; i < numSamples; i++)
      maxLoad = MAX2 (maxLoad, MAX3 (mesh[0][i], mesh[1][i], mesh[2][i]));
  }
  GetLoad (&mesh[0][curMeshPointer], &mesh[1][curMeshPointer], &mesh[2][curMeshPointer]);
  maxLoad = MAX2 (maxLoad, MAX3 (mesh[0][curMeshPointer], mesh[1][curMeshPointer], mesh[2][curMeshPointer]));
  makeLoad ();
  glutPostRedisplay ();
}
/* redraw, what else ? */
void 
redraw (void)
{

  if (newModel)
    recalcModelView ();
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  if(anim && !highqualitymoving){
    if (blending) {
      blending = 2;
      glDisable (GL_BLEND);
    }
    if (antialias) {
      antialias = 2;
      glDisable (GL_LINE_SMOOTH);
      glDisable (GL_POLYGON_SMOOTH);
    }
    makeLoad ();
  }else{
    if (blending == 2) {
      blending = 1;
      glEnable (GL_BLEND);
    }
    if (antialias == 2) {
      antialias = 1;
      glEnable (GL_LINE_SMOOTH);
      glEnable (GL_POLYGON_SMOOTH);
    }
    makeLoad ();
    glutPostRedisplay ();
  }
  glCallList (1);
  glutSwapBuffers ();
}
/* viewport size has changed */
void
myReshape (int w, int h)
{
  if (w==W && h!=H)       W=H=h;  
  else if (w!=W && h==H)  W=H=w;
  else                    W=H= w>h ? w : h;
  glutReshapeWindow ( W, H );
  glViewport (0, 0, W, H);
}
/* mouse events and handling */
void
mouse (int button, int state, int x, int y)
{
  if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN) {
    spinning = 0;
    anim = 0;
    moving = 1;
    beginx = x;
    beginy = y;
    if (glutGetModifiers () & GLUT_ACTIVE_SHIFT) {
      scaling = 1;
    } else {
      scaling = 0;
    }
  }
  if (button == GLUT_LEFT_BUTTON && state == GLUT_UP) {
    moving = 0;
  }
  redraw();
}
/* animation when idle, get new load from time to time */
void 
idleFunc (void)
{
  if (actualtime != time (0)) {
    actualtime = time (0);
    if (actualtime % updatespeed == 0) {
      getLoad ();
    }
  }
  if (anim)
    add_quats (lastquat, curquat, curquat);
    newModel = 1;
    glutPostRedisplay ();
#ifdef SYSV
  poll (NULL, (size_t) 0, 50);
#else
  usleep (50000L);		/* 5/100 sec */
#endif
}
/* motion function, called from glut, starts animation for spinning */
void
motion (int x, int y)
{
  if (scaling) {
    scalefactor = scalefactor * (1.0 + (((float) (beginy - y)) / H));
    beginx = x;
    beginy = y;
    newModel = 1;
    glutPostRedisplay ();
    return;
  }
  if (moving) {
    trackball (lastquat,
	       (2.0 * beginx - W) / W,
	       (H - 2.0 * beginy) / H,
	       (2.0 * x - W) / W,
	       (H - 2.0 * y) / H
      );
    beginx = x;
    beginy = y;
    spinning = 1;
    anim = 1;
  }
}
/* not spinning when window is hidden */
void
vis (int visible)
{
  if (visible == GLUT_VISIBLE) {
    if (spinning)
      anim = 1;
  } else {
    if (spinning)
      anim = 0;
  }
}
/* texture submenu */
void 
textureMenuFunc (int value)
{
  switch (value) {
  case 1:
    texturing = 1 - texturing;
    makeLoad ();
    break;
  case 2:
    xTextureRepeat *= 2;
    makeLoad ();
    break;
  case 3:
    xTextureRepeat /= 2;
    xTextureRepeat = MAX2 (xTextureRepeat, 1);
    makeLoad ();
    break;
  case 4:
    yTextureRepeat *= 2;
    makeLoad ();
    break;
  case 5:
    yTextureRepeat /= 2;
    yTextureRepeat = MAX2 (yTextureRepeat, 1);
    makeLoad ();
    break;
  }
}
/* main menu */
void 
controlMenu (int value)
{
  switch (value) {
  case 1:
    glutFullScreen ();
    break;
  case 2:
    lighting = 1 - lighting;
    if (lighting) {
      glEnable (GL_LIGHTING);
      glShadeModel (GL_SMOOTH);
    } else
      glDisable (GL_LIGHTING);
    makeLoad ();
    break;
  case 3:
    showLoad = 1 - showLoad;
    makeLoad ();
    break;
  case 5:
    showPlanes = 1 - showPlanes;
    makeLoad ();
    break;
  case 6:
    showClock = 1 - showClock;
    makeLoad ();
    break;
  case 7:
    showHost = 1 - showHost;
    makeLoad ();
    break;
  case 8:
    blending = 1 - blending;
    if (blending)
      glEnable (GL_BLEND);
    else
      glDisable (GL_BLEND);
    makeLoad ();
    break;
  case 9:
    highqualitymoving = 1 - highqualitymoving;
    break;
  case 10:
    antialias = 1 - antialias;
    if (antialias) {
      glEnable (GL_LINE_SMOOTH);
      glEnable (GL_POLYGON_SMOOTH);
    } else {
      glDisable (GL_LINE_SMOOTH);
      glDisable (GL_POLYGON_SMOOTH);
    }
    makeLoad ();
    break;
  case 11:
    screenShot("glload.tga",W,H);
    break;
  case 110:
    exit (0);
    break;
  }
  glutPostRedisplay ();
}
void 
usage ()
{
  char **cpp;

  fprintf (stderr, "\nusage:  %s [-options ...] \n", ProgName);
  for (cpp = help_message; *cpp; cpp++) {
    fprintf (stderr, "%s\n", *cpp);
  }
  fprintf (stderr, "\n");
  exit (1);
}


int
main (int argc, char **argv)
{
  GLint i, textureSize;
  int imageWidthIn, imageHeightIn, imageWidthOut, imageHeightOut;
  int textureMenu;
  float width2height;
  int height2width;
  char fileName[255];

  /* command line option parsing */
  ProgName = argv[0];
  for (i = 1; i < argc; i++) {
    char *arg = argv[i];

    if (arg[0] == '-') {
      switch (arg[1]) {
      case 'u':
	if (++i >= argc)
	  usage ();
	sscanf (argv[i], "%d", &updatespeed);
	continue;
      case 's':
	switch (arg[2]) {
	case 's':
	  if (++i >= argc)
	    usage ();
	  sscanf (argv[i], "%f", &smallStart);
	  continue;
	case 'e':
	  if (++i >= argc)
	    usage ();
	  sscanf (argv[i], "%f", &smallEnd);
	  continue;
	case 't':
	  if (++i >= argc)
	    usage ();
	  sscanf (argv[i], "%f", &smallText);
	  continue;
	default:
	  if (++i >= argc)
	    usage ();
	  sscanf (argv[i], "%d", &showLoad);
	  continue;
	}
	continue;
      case 'm':
	switch (arg[2]) {
	case 's':
	  if (++i >= argc)
	    usage ();
	  sscanf (argv[i], "%f", &mediumStart);
	  continue;
	case 'e':
	  if (++i >= argc)
	    usage ();
	  sscanf (argv[i], "%f", &mediumEnd);
	  continue;
	case 't':
	  if (++i >= argc)
	    usage ();
	  sscanf (argv[i], "%f", &mediumText);
	  continue;
	}
	continue;
      case 'l':
	switch (arg[2]) {
	case 's':
	  if (++i >= argc)
	    usage ();
	  sscanf (argv[i], "%f", &largeStart);
	  continue;
	case 'e':
	  if (++i >= argc)
	    usage ();
	  sscanf (argv[i], "%f", &largeEnd);
	  continue;
	case 't':
	  if (++i >= argc)
	    usage ();
	  sscanf (argv[i], "%f", &largeText);
	  continue;
	default:
	  if (++i >= argc)
	    usage ();
	  sscanf (argv[i], "%d", &lighting);
	  continue;
	}
	continue;
      case 'n':
	if (++i >= argc)
	  usage ();
	sscanf (argv[i], "%d", &numSamples);
	continue;
      case 'p':
	if (++i >= argc)
	  usage ();
	sscanf (argv[i], "%d", &showPlanes);
	continue;
      case 't':
	switch (arg[2]) {
	case 'x':
	  if (++i >= argc)
	    usage ();
	  sscanf (argv[i], "%d", &xTextureRepeat);
	  continue;
	case 'y':
	  if (++i >= argc)
	    usage ();
	  sscanf (argv[i], "%d", &yTextureRepeat);
	  continue;
	default:
	  if (++i >= argc)
	    usage ();
	  sscanf (argv[i], "%s", fileName);
	  texturing = 1;
	  continue;
	}
      case 'c':
	if (++i >= argc)
	  usage ();
	sscanf (argv[i], "%d", &showClock);
	continue;
      case 'h':
	if (++i >= argc)
	  usage ();
	sscanf (argv[i], "%d", &showHost);
	continue;
      case 'r':
	if (++i >= argc)
	  usage ();
	sscanf (argv[i], "%f", &redValue);
	continue;
      case 'b':
	if (++i >= argc)
	  usage ();
	sscanf (argv[i], "%d", &blending);
	if (blending)
	  glEnable (GL_BLEND);
	else
	  glDisable (GL_BLEND);
	if (blending == 2) {
	  blending = 1;
	  highqualitymoving = 1;
	}
	continue;
      case 'a':
	if (++i >= argc)
	  usage ();
	sscanf (argv[i], "%d", &antialias);
	if (antialias) {
	  glEnable (GL_LINE_SMOOTH);
	  glEnable (GL_POLYGON_SMOOTH);
	} else {
	  glDisable (GL_LINE_SMOOTH);
	  glDisable (GL_POLYGON_SMOOTH);
	}
	continue;
      default:
	usage ();
      }
    } else {
      fprintf (stderr, "\nInvalid argument: %s\n", arg);
      usage ();
    }
  }  

  /* initialisation */
  mesh[0] = (float *)calloc (numSamples, sizeof (float));
  mesh[1] = (float *)calloc (numSamples, sizeof (float));
  mesh[2] = (float *)calloc (numSamples, sizeof (float));
  glutInit (&argc, argv);
  glutInitDisplayMode (GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
  trackball (curquat, 0.0, 0.0, 0.0, 0.0);
  glutCreateWindow ("GLload");
  glutDisplayFunc (redraw);
  glutReshapeFunc (myReshape);
  glutVisibilityFunc (vis);
  glutMouseFunc (mouse);
  glutMotionFunc (motion);
  glutIdleFunc (idleFunc);
  textureMenu = glutCreateMenu (textureMenuFunc);
  glutAddMenuEntry ("Toggle texturing", 1);
  glutAddMenuEntry ("Increase x texture repeat", 2);
  glutAddMenuEntry ("Decrease x texture repeat", 3);
  glutAddMenuEntry ("Increase y texture repeat", 4);
  glutAddMenuEntry ("Decrease y texture repeat", 5);
  glutCreateMenu (controlMenu);
  glutAddMenuEntry ("Full screen", 1);
  glutAddMenuEntry ("Toggle lighting", 2);
  glutAddMenuEntry ("Toggle load text", 3);
  if (texturing)
    glutAddSubMenu ("Texture", textureMenu);
  glutAddMenuEntry ("Toggle planes", 5);
  glutAddMenuEntry ("Toggle clock", 6);
  glutAddMenuEntry ("Toggle hostname", 7);
  glutAddMenuEntry ("Toggle alpha blending", 8);
  glutAddMenuEntry ("Toggle high quality moving", 9);
  glutAddMenuEntry ("Toggle antialiasing", 10);
  glutAddMenuEntry ("Screenshot to glload.tga", 11);
  glutAddMenuEntry ("Quit", 110);
  glutAttachMenu (GLUT_RIGHT_BUTTON);
#ifndef SVR4
  if (gethostname (hostname, 255) != 0) {
    perror ("gethostname");
    exit (10);
  }
#else
  if (sysinfo (SI_HOSTNAME, hostname, 255) < 0) {
    perror ("sysinfo(SI_HOSTNAME)");
    exit (10);
  }
#endif
  glEnable (GL_BLEND);
  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glEnable (GL_LINE_SMOOTH);
  glEnable (GL_POLYGON_SMOOTH);
  glDisable (GL_CULL_FACE);
  glEnable (GL_DEPTH_TEST);
  if (lighting) {
    glEnable (GL_LIGHTING);
    glShadeModel (GL_SMOOTH);
  } else
    glDisable (GL_LIGHTING);
  glMatrixMode (GL_PROJECTION);
  gluPerspective ( /* field of view in degree */ 40.0,
  /* aspect ratio */ 1.0,
		  /* Z near */ 1.0, /* Z far */ 50.0);
  glMatrixMode (GL_MODELVIEW);
  gluLookAt (-1.0, 0.0, 33.0,	/* eye is at (0,0,30) */
	     0.0, 0.0, 0.0,	/* center is at (0,0,0) */
	     0.0, 1.0, 0.);	/* up is in positive Y direction */
  glPushMatrix ();		/* dummy push so we can pop on model
				   recalc */
  glLightModeli (GL_LIGHT_MODEL_LOCAL_VIEWER, 1);
  glLightfv (GL_LIGHT0, GL_POSITION, lightZeroPosition);
  glLightfv (GL_LIGHT0, GL_DIFFUSE, lightZeroColor);
  glLightf (GL_LIGHT0, GL_CONSTANT_ATTENUATION, 0.1);
  glLightf (GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.05);
  glLightfv (GL_LIGHT1, GL_POSITION, lightOnePosition);
  glLightfv (GL_LIGHT1, GL_DIFFUSE, lightOneColor);
  glEnable (GL_LIGHT0);
  glEnable (GL_LIGHT1);
  glLineWidth (1.0);
  /* load and scale texture */
  if (texturing) {
    imgLoad (fileName, &imageWidthIn, &imageHeightIn, &imageDataIn);
#ifdef GL_VERSION_1_1
    glGetIntegerv (GL_PROXY_TEXTURE_2D, &textureSize);
    textureSize = sqrt (textureSize);
    for (i = 1; i < textureSize; i *= 2);
    textureSize = i / 2;
    if (textureSize == 0) {
      glGetIntegerv (GL_MAX_TEXTURE_SIZE, &textureSize);
      textureSize = sqrt (textureSize);
    }
#else
    glGetIntegerv (GL_MAX_TEXTURE_SIZE, &textureSize);
    textureSize = sqrt (textureSize);
#endif
    width2height = (int) ((int) imageWidthIn / (int) imageHeightIn);
    height2width = imageHeightIn / imageWidthIn;
    width2height = (int) sqrt (width2height);
    height2width = (int) sqrt (height2width);
    if (height2width > width2height) {
      i = 1;
      while (i < height2width)
        i *= 2;
      height2width = i;
      width2height = 1.0 / (float) height2width;
    } else {
      i = 1;
      while (i < width2height)
        i *= 2;
      width2height = i;
    }
    imageWidthOut = textureSize * width2height;
    imageHeightOut = textureSize / width2height;
    for (; (imageWidthOut / 2) >= imageWidthIn; imageWidthOut /= 2);
    for (; (imageHeightOut / 2) >= imageHeightIn; imageHeightOut /= 2);
    if (imageWidthIn != imageWidthOut || imageHeightIn != imageHeightOut) {
      imageDataOut = (GLubyte *)malloc (imageWidthOut * imageHeightOut * 4);
      gluScaleImage (GL_RGBA, imageWidthIn, imageHeightIn,
		     GL_UNSIGNED_BYTE, imageDataIn,
		     imageWidthOut, imageHeightOut,
		     GL_UNSIGNED_BYTE, imageDataOut);
      printf ("%s: Scaled image %s from %dx%d to %dx%d\n",
 	    argv[0], fileName, imageWidthIn, imageHeightIn, imageWidthOut, imageHeightOut);
   } else {
      imageDataOut = imageDataIn;
      imageWidthOut = imageWidthIn;
      imageHeightOut = imageHeightIn;
    }
    glTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexImage2D (GL_TEXTURE_2D, 0, 4,
		  imageWidthOut, imageHeightOut,
		  0, GL_RGBA, GL_UNSIGNED_BYTE, imageDataOut);
  }
  /* init mesh array with current load */
  GetLoad (&mesh[0][0], &mesh[1][0], &mesh[2][0]);
  for (i = 1; i < numSamples; i++) {
    mesh[0][i] = mesh[0][0];
    mesh[1][i] = mesh[1][0];
    mesh[2][i] = mesh[2][0];
  }
  glutMainLoop ();
  return 0;			/* ANSI C requires main to return int. */
}
/* 
 * $Log: glload.c,v $
 * Revision 1.7  2000/03/08 13:03:53  eilemann
 * *** empty log message ***
 *
 */
